@@ -2,12 +2,13 @@ import base64
 import calendar
 import datetime
 import hashlib
+import re
 import time
 import urllib
 import uuid
 
 from py4web import redirect, request, response, abort, URL, action
-from py4web.core import Fixture, Template, REX_APPJSON
+from py4web.core import Fixture, Template, REGEX_APPJSON
 from pydal.validators import (
     IS_EMAIL,
     CRYPT,
@@ -53,11 +54,11 @@ class AuthEnforcer(Fixture):
     def transform(self, output, shared_data):
         return self.auth.transform(output, shared_data)
 
-    def abort_or_redirect(self, page, message=''):
+    def abort_or_redirect(self, page, message=""):
         """
         return HTTP 403 if 'application/json' in HTTP_ACCEPT
         else redirects to page"""
-        if REX_APPJSON.search(request.headers.get("accept", "")):
+        if re.search(REGEX_APPJSON, request.headers.get("accept", "")):
             abort(403)
         redirect_next = request.fullpath
         if request.query_string:
@@ -126,7 +127,7 @@ class Auth(Fixture):
         login_expiration_time=3600,  # seconds
         password_complexity={"entropy": 50},
         block_previous_password_num=None,
-        allowed_actions=['all'],
+        allowed_actions=["all"],
         use_appname_in_redirects=True,
     ):
         """Creates and Auth object responsinble for handling
@@ -159,7 +160,7 @@ class Auth(Fixture):
 
     def transform(self, output, shared_data):
         if self.inject:
-            template_context = shared_data.get('template_context')
+            template_context = shared_data.get("template_context")
             template_context["user"] = self.get_user()
         return output
 
@@ -220,7 +221,13 @@ class Auth(Fixture):
                 )
             if self.block_previous_password_num is not None:
                 auth_fields.append(
-                    Field("past_passwords_hash", "list:string", writable=False, readable=False))
+                    Field(
+                        "past_passwords_hash",
+                        "list:string",
+                        writable=False,
+                        readable=False,
+                    )
+                )
             db.define_table("auth_user", *auth_fields, *self.extra_auth_user_fields)
 
     @property
@@ -341,7 +348,7 @@ class Auth(Fixture):
                     ]
                     return {
                         "allowed_actions": self.allowed_actions,
-                        "plugins": ['local'] + [key for key in self.plugins],
+                        "plugins": ["local"] + [key for key in self.plugins],
                         "fields": fields,
                     }
                 # Otherwise, we assume the user exists.
@@ -373,7 +380,9 @@ class Auth(Fixture):
                             if self.db:
                                 data = self.get_or_register_user(data)
                                 self.session["user"] = {"id": data["id"]}
-                                self.session["recent_activity"] = calendar.timegm(time.gmtime())
+                                self.session["recent_activity"] = calendar.timegm(
+                                    time.gmtime()
+                                )
                                 self.session["uuid"] = str(uuid.uuid1())
                         else:
                             data = self._error("Invalid Credentials")
@@ -382,7 +391,9 @@ class Auth(Fixture):
                         user, error = self.login(**vars)
                         if user:
                             self.session["user"] = {"id": user.id}
-                            self.session["recent_activity"] = calendar.timegm(time.gmtime())
+                            self.session["recent_activity"] = calendar.timegm(
+                                time.gmtime()
+                            )
                             self.session["uuid"] = str(uuid.uuid1())
                             user = {
                                 f.name: user[f.name]
@@ -564,7 +575,9 @@ class Auth(Fixture):
                     }
                 }
             if self.block_previous_password_num:
-                past_pwds = (user.past_passwords_hash or [])[: self.block_previous_password_num]
+                past_pwds = (user.past_passwords_hash or [])[
+                    : self.block_previous_password_num
+                ]
                 if any(new_pwd == old_pwd for old_pwd in past_pwds):
                     return {"errors": {"new_password": "new password was already used"}}
                 else:
@@ -573,7 +586,7 @@ class Auth(Fixture):
         num = db(db.auth_user.id == user.id).update(
             password=new_pwd, last_password_change=datetime.datetime.utcnow()
         )
-        return {'updated': num}
+        return {"updated": num}
 
     def change_email(self, user, new_email, password=None, check=True):
         db = self.db
@@ -667,3 +680,56 @@ class Auth(Fixture):
             print('Mock send to %s subject "%s" body:\n%s\n' % (email, subject, body))
             return True
         return self.sender.send(email, subject=subject, body=body)
+
+    def enable_record_versioning(
+        self,
+        tables,
+        archive_db=None,
+        archive_names="%(tablename)s_archive",
+        current_record="current_record",
+        current_record_label=None,
+    ):
+        """
+        Used to enable full record versioning (including auth tables)::
+
+            auth = Auth(db)
+            auth.define_tables()
+            # define our own tables
+            db.define_table(
+                'mything',
+                Field('name'),
+                auth.signature())
+            auth.enable_record_versioning(tables=db)
+
+        tables can be the db (all table) or a list of tables.
+        only tables with modified_by and modified_on fiels (as created
+        by auth.signature) will have versioning. Old record versions will be
+        in table 'mything_archive' automatically defined.
+        when you enable enable_record_versioning, records are never
+        deleted but marked with is_active=False.
+
+        enable_record_versioning enables a common_filter for
+        every table that filters out records with is_active = False
+
+        Note:
+            If you use auth.enable_record_versioning,
+            do not use auth.archive or you will end up with duplicates.
+            auth.archive does explicitly what enable_record_versioning
+            does automatically.
+        """
+        current_record_label = (
+            current_record_label or current_record.replace("_", " ").title()
+        )
+        for table in tables:
+            fieldnames = table.fields()
+            if (
+                "id" in fieldnames
+                and "modified_on" in fieldnames
+                and current_record not in fieldnames
+            ):
+                table._enable_record_versioning(
+                    archive_db=archive_db,
+                    archive_name=archive_names,
+                    current_record=current_record,
+                    current_record_label=current_record_label,
+                )
